// Copyright 2010-2012 Kevin Gimpel
// Licensed under the modified BSD license. See the LICENSE file included with this distribution.

#ifndef _UTIL_H
#define	_UTIL_H

#include <vector>
#include <string>
#include <sstream>
#include <fstream>
#include <sys/stat.h>
#include <sys/time.h>

using namespace std;

extern const double neg_inf;
extern const double pos_inf;
extern const float neg_inf_float;
extern const float pos_inf_float;

// Returns the number of times that c occurs in s.
int countChar(const string& s, char c);

vector<string> tokenize(const string& str,const string& delimiters);

void tokenize(const string& str, char delimiter, vector<string>& tokens);

int splitString(const string& input, const string& delimiter, vector<string>& results, bool includeEmpties = false);

double logAdd(const double lx, const double ly);

// Converts int to string.
string itos(int i);

// Returns log(x!), computed in the log-domain.
double logFactorial(int x);

// Returns "n choose k".
// Note: if overflow is experienced, change the implementation
// to use the log domain or write a new version of this function
// that uses the log domain.
int nchoosek(int n, int k);

// Check if a file exists.
bool fileExists(string strFilename);

// Reads all lines from given file, placing them in the given vector.
void readLinesFromFile(const string& inputFile, vector<string>& lines);

// Computes time2 - time1, returning the value in seconds.
double computeTimeDifference(struct timeval& time1, struct timeval& time2);

// Computes the transpose of the given two-dimensional vector, writing the result into output.
// output is not initially cleared.
void rotateTwoDimensionalVector(const vector< vector<string> >& input, vector< vector<string> >& output);

double min(double a, double b);

string trimString(const string& str);

int getRandomInteger(int max);

#endif	/* _UTIL_H */

